CIO_SISO = "ss"
CIO_SIFO = "sf"
CIO_FISO = "fs"
CIO_FIFO = "ff"
CIO_Types = [CIO_SISO, CIO_SIFO, CIO_FISO, CIO_FIFO]

defaultIO = CIO_SISO

defaultTimeLimit = 5

defaultExecutors = {
    "c": ["gcc {fileName} -o {fileNameWithoutExt}", "./{fileNameWithoutExt}"],
    "cpp": ["g++ {fileName} -o {fileNameWithoutExt}", "./{fileNameWithoutExt}"],
    "java": ["javac {fileName}", "java {fileNameWithoutExt}"],
    "python": ["python {fileName}"],
    "pascal": ["fpc {fileName}", "./{fileNameWithoutExt}"],
    "objective-c": ["gcc -framework Cocoa {fileName} -o {fileNameWithoutExt}", "./{fileNameWithoutExt}"],
    "javascript": ["node {fileName}"],
    "ruby": ["ruby {filename}"],
    "go": ["go run {filename}"],
    "shellscript": ["bash {filename}"],
    "powershell": ["powershell -ExecutionPolicy ByPass -File {filename}"]
}

defaultTempFileFilter = ["exe", "o", "class", "out"]


defaultImportedCommand = {
    "ls": "ls",
    "dir": "dir",
    "cls": "clear",
    "cat": "cat",
    "mkdir": "mkdir",
    "echo": "echo",
    "copy": "copy",
    "cp": "cp",
    "del": "del",
    "diff": "diff",
    "move": "move",
    "mv": "mv",
    "rm": "rm",
    "rmdir": "rmdir",
    "gcc": "gcc",
    "g++": "g++",
    "java": "java",
    "javac": "javac",
    "python": "python",
    "fpc": "fpc",
    "node": "node",
    "ruby": "ruby",
    "go": "go",
    "bash": "bash",
    "powershell": "powershell",
    "git": "git",
    "vim": "vim",
    "gdb": "gdb",
}

defaultCodeTemplate = {
    "c":
    """#include <stdio.h>
int main()
{
    return 0;
}""",
    "cpp":
    """#include <cstdio>
#include <cstdlib>
#include <iostream>
using namespace std;
int main()
{

    return EXIT_SUCCESS;
} """,
    "java": """import java.util.Scanner;

public class Main {
    public static void main(String[] args) {
        
    }
}
""",
    "python": """def main():

    return 0

if __name__ == "__main__":
    exit(main())

""",
    "pascal": """program pro(Input, Output);
var

begin
    
end.
""",
}



from ecr.core.manager import getConfigPath, getExecutorPath, getFileExt, getFileInputPath, getFileOutputPath, getMainPath, getTemplatePath
import os


class TestPath:
    @classmethod
    def setup_class(cls):
        cls.unixPath = "~/work"
        cls.winPath = r"C:\work"

    def _test_end(self, end, func):
        assert func(self.unixPath).endswith(end)
        assert func(self.winPath).endswith(end)

    def test_config(self):
        self._test_end("config.json", getConfigPath)

    def test_executor(self):
        self._test_end("executor.json", getExecutorPath)

    def test_templates(self):
        self._test_end("templates", getTemplatePath)

    def test_fileinput(self):
        self._test_end("input.data", getFileInputPath)

    def test_fileoutput(self):
        self._test_end("output.data", getFileOutputPath)
    
    def test_fileext(self):
        assert getFileExt("~/dir/a.c") == "c"
        assert getFileExt("~/dir/a.java") == "java"
        assert getFileExt(r"C:\a.java") == "java"
        assert getFileExt(r"D:\p.pas") == "pas"



import argparse
import sys
import io
import os
import platform
import codecs
import locale
import subprocess
import shlex
from enum import Enum
from prompt_toolkit.styles import Style
from .core import manager
from .ui import color, cli
from .ui.cli import console

version = "0.0.1.4"


class ReturnCode(Enum):
    OK = 0
    ERROR = -1
    UNLOADED = 1
    RUNERR = 2


cwd = None

man: manager.WorkManager = None

itParser = None


def loadMan():
    global man
    try:
        man = manager.load(cwd)
    except:
        man = None


def printHead():
    assert(man == None or man.state != manager.WorkManagerState.Empty)
    if man == None:
        console.write("ECR", end=" ")
    elif man.state == manager.WorkManagerState.Loaded:
        console.write(color.useGreen("ECR"), end=" ")
    elif man.state == manager.WorkManagerState.LoadFailed:
        console.write(color.useRed("ECR"), end=" ")
    console.write(cwd)


def assertInited()->bool:
    if man == None:
        console.error("Not have any ecr directory")
        return False
    return True


def init(args):
    global man
    manager.initialize(cwd)
    loadMan()
    printHead()
    return ReturnCode.OK if man != None else ReturnCode.UNLOADED


def now(args):
    if not assertInited():
        return ReturnCode.UNLOADED
    man.currentFile = args.path
    return ReturnCode.OK


def new(args):
    if not assertInited():
        return ReturnCode.UNLOADED
    man.newCode(args.filename)
    man.currentFile = args.filename
    return ReturnCode.OK


def run(args):
    if not assertInited():
        return ReturnCode.UNLOADED

    if args.file == None and man.currentFile == None:
        console.write("Please set file first")
        return ReturnCode.ERROR

    result = False

    result = man.execute(io=args.io, file=args.file)

    if not result:
        console.error("Running Failed")
        return ReturnCode.RUNERR
    return ReturnCode.OK


def clean(args):
    if not assertInited():
        return ReturnCode.UNLOADED
    man.clean()
    return ReturnCode.OK


def shutdown(args):
    exit(0)


def pwd(args):
    console.write(cwd)
    return ReturnCode.OK


def getVersion(args):
    console.write("edl-cr", version)
    return ReturnCode.OK


def cd(args):
    global man, cwd
    if not os.path.exists(args.path):
        console.error("No this directory")
        return ReturnCode.ERROR
    os.chdir(args.path)
    cwd = os.getcwd()
    printHead()
    if manager.hasInitialized(cwd):
        loadMan()
    return ReturnCode.OK


def clear(args):
    global man
    if not assertInited():
        return ReturnCode.UNLOADED
    if console.confirm("Do you want to clear ALL?", [cli.SwitchState.OK, cli.SwitchState.Cancel]) == cli.SwitchState.OK:
        manager.clear(man.workingDirectory)
        man = None
    return ReturnCode.OK


class BasicError(Exception):
    def __init__(self, message):
        self.message = message

    def __str__(self):
        return self.message


class ITParser(argparse.ArgumentParser):
    def error(self, message):
        raise BasicError(message)

    def exit(self, status=0, message=None):
        if message:
            self._print_message(message, sys.stderr)


builtinCmdLists = ["init", "new", "now", "pwd",
                   "cd", "clear", "run", "clean", "version", "exit"]
cmdLists = None


def getITParser():
    parser = ITParser(
        prog="", description="Code Runner", add_help=True)

    subpars = parser.add_subparsers()

    cmd_init = subpars.add_parser("init", help="Initialize ECR data")
    cmd_init.set_defaults(func=init)

    cmd_new = subpars.add_parser("new", help="Create new code file")
    cmd_new.add_argument("filename")
    cmd_new.set_defaults(func=new)

    cmd_now = subpars.add_parser("now", help="Change current file")
    cmd_now.add_argument("path", nargs="?", default=None,
                         help="Set current file (clear for none)")
    cmd_now.set_defaults(func=now)

    cmd_pwd = subpars.add_parser("pwd", help="Print working directory")
    cmd_pwd.set_defaults(func=pwd)

    cmd_cd = subpars.add_parser("cd", help="Change working directory")
    cmd_cd.add_argument("path")
    cmd_cd.set_defaults(func=cd)

    cmd_clear = subpars.add_parser("clear", help="Clear ECR data")
    cmd_clear.set_defaults(func=clear)

    cmd_run = subpars.add_parser("run", help="Run code file")
    cmd_run.add_argument("-io", "--io", choices=manager.CIO_Types,
                         default=None, help="Change input and output")
    cmd_run.add_argument(
        "file", nargs="?", default=None, help="File name (only for this command)")
    cmd_run.set_defaults(func=run)

    cmd_clean = subpars.add_parser("clean", help="Clean temp files")
    cmd_clean.set_defaults(func=clean)

    cmd_version = subpars.add_parser("version", help="Get version")
    cmd_version.set_defaults(func=getVersion)

    # cmd_help = subpars.add_parser("help", help="Help")
    # cmd_help.set_defaults(func=gethelp)

    cmd_exit = subpars.add_parser("exit", help="Exit")
    cmd_exit.set_defaults(func=shutdown)

    return parser


def doSyscall(cmd, message):
    console.info(message, end=" ")
    console.write(cmd)
    retCode = os.system(manager.getSystemCommand(cmd, man))
    console.info(f"System command exited:", end=" ")
    if retCode == 0:
        console.write(retCode)
    else:
        console.error(retCode)
    return retCode


defaultPrompt = "> "


def mainInit():
    global itParser, cmdLists, cwd
    from prompt_toolkit import PromptSession
    from pygments.lexers.shell import BashLexer
    from prompt_toolkit.lexers import PygmentsLexer
    from prompt_toolkit.auto_suggest import AutoSuggestFromHistory

    itParser = getITParser()
    cwd = os.getcwd()

    if manager.hasInitialized(cwd):
        loadMan()

    cmdLists = list(builtinCmdLists)
    if man != None:
        cmdLists += man.importedCommand.keys()

    cliInputSession = PromptSession(
        message=defaultPrompt, lexer=PygmentsLexer(BashLexer), auto_suggest=AutoSuggestFromHistory())
    cli.console = cli.CLI(inputCommandSession=cliInputSession)


def executeCommand(oricmd):
    cargs = shlex.split(oricmd)
    if len(cargs) == 0:
        return
    if cargs[0].startswith(">"):
        return doSyscall(oricmd[1:], "Call system command:")
    else:
        try:
            cmd = itParser.parse_args(cargs)
        except BasicError as e:  # when parse failed, parser will call exit()
            if man != None and cargs[0] in man.importedCommand:
                return doSyscall(
                    f"{man.importedCommand[cargs[0]]} {' '.join(cargs[1:])}", "Imported command:")
            else:
                console.warning("We can't recognize this command:")
                console.write(f"  {e}")
                if console.confirm("Do you mean a system command?", [cli.SwitchState.Yes, cli.SwitchState.No]) == cli.SwitchState.Yes:
                    return doSyscall(oricmd, "Call system command:")
        else:
            if hasattr(cmd, "func"):
                return cmd.func(cmd).value
            else:
                return 0


def getCommandCompleter():
    from prompt_toolkit.completion import WordCompleter, merge_completers
    wc = WordCompleter(cmdLists, ignore_case=True)
    pc = cli.PathCompleter()
    return merge_completers([wc, pc])


def main():  # pragma: no cover
    global man, cwd, itParser

    baseParser = argparse.ArgumentParser(
        prog="ecr", description="Code Runner")
    baseParser.add_argument("-v", "--version", default=False, action="store_true",
                            help="Get version")
    baseParser.add_argument("-w", "--wdir", default=None,
                            help="Set working directory")
    baseParser.add_argument(
        "-c", "--command", default=None,  help="Execute command")
    baseCmd = baseParser.parse_args()
    if baseCmd.version:
        return getVersion(None).value
    if baseCmd.wdir != None:
        os.chdir(baseCmd.wdir)

    mainInit()

    if baseCmd.command != None:
        return executeCommand(baseCmd.command)

    printHead()
    while True:
        try:
            oricmd = str(console.inputCommand(
                f'{man.currentFile if man != None and man.currentFile != None else ""}{defaultPrompt}', completer=getCommandCompleter(), complete_in_thread=True))
        except KeyboardInterrupt:
            continue
        except EOFError:
            break
        executeCommand(oricmd)

    return 0


def outmain():  # pragma: no cover
    exit(int(main()))


if __name__ == "__main__":  # pragma: no cover
    outmain()



import color
import os
import platform
from prompt_toolkit import prompt, print_formatted_text, HTML, PromptSession
from pygments.lexers.shell import BashLexer
from prompt_toolkit.lexers import PygmentsLexer
from prompt_toolkit.completion import WordCompleter, Completer, Completion
from prompt_toolkit.auto_suggest import AutoSuggestFromHistory
from prompt_toolkit.shortcuts import ProgressBar
from enum import Enum


class PathCompleter(Completer):
    """
    Complete for Path variables.

    :param get_paths: Callable which returns a list of directories to look into
                      when the user enters a relative path.
    :param file_filter: Callable which takes a filename and returns whether
                        this file should show up in the completion. ``None``
                        when no filtering has to be done.
    :param min_input_len: Don't do autocompletion when the input string is shorter.
    """

    def __init__(self, only_directories=False, get_paths=None, file_filter=None,
                 min_input_len=0, expanduser=False):
        assert get_paths is None or callable(get_paths)
        assert file_filter is None or callable(file_filter)
        assert isinstance(min_input_len, int)
        assert isinstance(expanduser, bool)

        self.only_directories = only_directories
        self.get_paths = get_paths or (lambda: ['.'])
        self.file_filter = file_filter or (lambda _: True)
        self.min_input_len = min_input_len
        self.expanduser = expanduser

    def get_completions(self, document, complete_event):
        text = document.text_before_cursor.split()[-1]

        # Complete only when we have at least the minimal input length,
        # otherwise, we can too many results and autocompletion will become too
        # heavy.
        if len(text) < self.min_input_len:
            return

        try:
            # Do tilde expansion.
            if self.expanduser:
                text = os.path.expanduser(text)

            # Directories where to look.
            dirname = os.path.dirname(text)
            if dirname:
                directories = [os.path.dirname(os.path.join(p, text))
                               for p in self.get_paths()]
            else:
                directories = self.get_paths()

            # Start of current file.
            prefix = os.path.basename(text)

            # Get all filenames.
            filenames = []
            for directory in directories:
                # Look for matches in this directory.
                if os.path.isdir(directory):
                    for filename in os.listdir(directory):
                        if filename.startswith(prefix):
                            filenames.append((directory, filename))

            # Sort
            filenames = sorted(filenames, key=lambda k: k[1])

            # Yield them.
            for directory, filename in filenames:
                completion = filename[len(prefix):]
                full_name = os.path.join(directory, filename)

                if os.path.isdir(full_name):
                    # For directories, add a slash to the filename.
                    # (We don't add them to the `completion`. Users can type it
                    # to trigger the autocompletion themselves.)
                    filename += '/'
                elif self.only_directories:
                    continue

                if not self.file_filter(full_name):
                    continue

                yield Completion(completion, 0, display=filename)
        except OSError:
            pass


class SwitchState(Enum):
    Yes = 0
    No = 1
    OK = 2
    Cancel = 3


confirmStrToSwitch = {
    "y": SwitchState.Yes,
    "n": SwitchState.No,
    "o": SwitchState.OK,
    "c": SwitchState.Cancel
}

switchToConfirmStr = {v: k for k, v in confirmStrToSwitch.items()}

defaultInputCommandSession = PromptSession(
    message="> ", lexer=PygmentsLexer(BashLexer), auto_suggest=AutoSuggestFromHistory())


class CLI:
    def __init__(self, inputCommandSession: PromptSession = None):
        self.write = print_formatted_text
        self.read = prompt
        self.getProgressBar = ProgressBar
        self.inputCommandSession = inputCommandSession if inputCommandSession != None else defaultInputCommandSession
        self.inputCommand = self.inputCommandSession.prompt

    def clear(self):
        if platform.system() == "Windows":
            os.system("cls")
        else:
            os.system("clear")

    def info(self, message, end="\n"):
        self.write(color.useCyan(message), end=end)

    def warning(self, message, end="\n"):
        self.write(color.useYellow(message), end=end)

    def error(self, message, end="\n"):
        self.write(color.useRed(message), end=end)

    def ok(self, message, end="\n"):
        self.write(color.useGreen(message), end=end)

    def confirm(self, message, choice):  # pragma: no cover
        swstr = ','.join([switchToConfirmStr[x] for x in choice])
        self.write(message, f"({swstr})", end=" ")
        ret = self.read()
        while ret not in confirmStrToSwitch or confirmStrToSwitch[ret] not in choice:
            self.write(
                f"Not an acceptable value. Please input again ({swstr}):", end=" ")
            ret = self.read()
        return confirmStrToSwitch[ret]


console = CLI()



from ecr import __main__ as main
from ecr.core import manager
import os
import shutil
from argparse import Namespace


class TestCLI:
    @classmethod
    def setup_class(cls):
        cls.oldCwd = os.getcwd()
        pat = os.path.dirname(__file__)
        while not "src" in os.listdir(pat):
            pat = os.path.dirname(pat)
        pat = os.path.join(pat, "temp")
        if not os.path.isdir(pat):
            os.mkdir(pat)
        assert os.path.isdir(pat)
        cls.path = pat
        cls.mpath = os.path.join(cls.path, "testC")
        if os.path.isdir(cls.mpath):
            shutil.rmtree(cls.mpath)
        os.mkdir(cls.mpath)
        assert os.path.isdir(cls.mpath)
        os.chdir(cls.mpath)
        main.mainInit()
        main.printHead()
        assert main.executeCommand("init") == 0
        assert main.man != None and main.man.state == manager.WorkManagerState.Loaded

    @classmethod
    def teardown_class(cls):
        try:
            shutil.rmtree(cls.mpath)
        except:
            pass
        os.chdir(cls.oldCwd)

    def test_basiccmd(self):
        assert main.executeCommand("pwd") == 0
        assert main.executeCommand("cd .") == 0
        # assert main.executeCommand("--help") == 0
        assert main.executeCommand("init -h") == 0

    def test_newcode(self):
        assert main.executeCommand("new 1.cpp") == 0
        assert os.path.isfile(os.path.join(self.mpath, "1.cpp"))

    def test_execute(self):
        _file = "1.py"
        fpath = os.path.join(self.mpath, _file)
        assert main.executeCommand(f"new {_file}") == 0
        assert os.path.isfile(fpath)
        main.man.currentFile = None
        main.executeCommand(f"run")
        main.executeCommand(f"run {_file}")
        main.executeCommand(f"now {_file}")
        assert main.man.currentFile == _file
        main.man.defaultTimeLimit = 0.5
        main.executeCommand(f"run {_file} -io ff")
        with open(fpath, "w") as f:
            f.write("while True: pass")
        main.executeCommand(f"run {_file} -io ff")

    def test_clean(self):
        main.executeCommand("new 1.cpp")
        main.executeCommand("run 1.cpp")
        assert main.executeCommand("clean") == 0

    def test_clear(self):
        pass

    def test_syscall(self):
        assert main.executeCommand("ls") == 0
        assert main.executeCommand(">ls") == 0
        # assert main.executeCommand("cls") == 0



from ecr.core import manager
import os
import shutil


class TestManager:
    @classmethod
    def setup_class(cls):
        pat = os.path.dirname(__file__)
        while not "src" in os.listdir(pat):
            pat = os.path.dirname(pat)
        pat = os.path.join(pat, "temp")
        if not os.path.isdir(pat):
            os.mkdir(pat)
        assert os.path.isdir(pat)
        cls.path = pat
        cls.mpath = os.path.join(cls.path, "testM")
        if os.path.isdir(cls.mpath):
            shutil.rmtree(cls.mpath)
        os.mkdir(cls.mpath)
        assert os.path.isdir(cls.mpath)
    
    @classmethod
    def teardown_class(cls):
        shutil.rmtree(cls.mpath)

    def test_init(self):
        manager.initialize(self.mpath)
        epath = manager.getMainPath(self.mpath)
        assert manager.hasInitialized(self.mpath)
        assert os.path.isdir(epath)
        assert os.path.isdir(manager.getTemplatePath(self.mpath))
        assert os.path.isfile(manager.getExecutorPath(self.mpath))
        assert os.path.isfile(manager.getConfigPath(self.mpath))
        assert os.path.isfile(manager.getFileInputPath(self.mpath))
        assert os.path.isfile(manager.getFileOutputPath(self.mpath))

    def test_load(self):
        if not manager.hasInitialized(self.mpath):
            self.test_init()
        self.man = manager.load(self.mpath)
        assert self.man.workingDirectory == self.mpath
        assert self.man.state == manager.WorkManagerState.Loaded

    def test_newcode(self):
        if not manager.hasInitialized(self.mpath):
            self.test_init()
        self.man = manager.load(self.mpath)
        self.man.newCode("1.cpp")
        assert os.path.isfile(os.path.join(self.mpath, "1.cpp"))
        self.man.newCode("2.c")
        assert os.path.isfile(os.path.join(self.mpath, "2.c"))
        self.man.newCode("1.cpp")
        assert os.path.isfile(os.path.join(self.mpath, "1.cpp"))
        self.man.newCode("3.txt")
        assert os.path.isfile(os.path.join(self.mpath, "3.txt"))
        
    
    def test_execute(self):
        if not manager.hasInitialized(self.mpath):
            self.test_init()
        self.man = manager.load(self.mpath)
        _file = "1.py"
        fpath = os.path.join(self.mpath, _file)
        self.man.newCode(_file)
        assert os.path.isfile(fpath)
        self.man.execute(file=_file)
        self.man.currentFile = _file
        self.man.defaultTimeLimit = 0.5
        self.man.execute()
        self.man.execute(io="ss")
        self.man.execute(io="ff")
        self.man.execute(io="fs")
        self.man.execute(io="sf")
        with open(fpath, "w") as f:
            f.write("while True: pass")
        self.man.execute()
    
    def test_clean(self):
        if not manager.hasInitialized(self.mpath):
            self.test_init()
        self.man = manager.load(self.mpath)
        self.man.newCode("1.cpp")
        self.man.execute(file="1.cpp")
        self.man.clean()

    def test_clear(self):
        assert manager.hasInitialized(self.mpath)
        manager.clear(self.mpath)
        assert not manager.hasInitialized(self.mpath)



import os
import json
import shutil
import re
import subprocess
import time
import platform
from enum import Enum
from ..ui import color, cli
from .defaultData import CIO_Types, CIO_SISO, CIO_FIFO, CIO_FISO, CIO_SIFO, defaultCodeTemplate, defaultExecutors, defaultImportedCommand, defaultIO, defaultTempFileFilter, defaultTimeLimit

CONST_tempFileFilter = "tempFileFilter"
CONST_importedCommand = "importedCommand"
CONST_defaultShell = "defaultShell"
CONST_defaultIO = "defaultIO"
CONST_defaultTimeLimit = "defaultTimeLimit"

fileextToLanguage = {
    "c": "c",
    "cpp": "cpp",
    "py": "python",
    "java": "java",
    "pas": "pascal",
    "m": "objective-c",
    "js": "javascript",
    "rb": "ruby",
    "go": "go",
    "sh": "shellscript",
    "ps1": "powershell",
}

languageToFileext = {v: k for k, v in fileextToLanguage.items()}

TEMPLATE_NAME = "base"


def getMainPath(basepath: str)->str:
    return os.path.join(basepath, ".ecr")


def getConfigPath(basepath: str) -> str:
    return os.path.join(getMainPath(basepath), "config.json")


def getExecutorPath(basepath: str) -> str:
    return os.path.join(getMainPath(basepath), "executor.json")


def getTemplatePath(basepath: str) -> str:
    return os.path.join(getMainPath(basepath), "templates")


def getFileInputPath(basepath: str) -> str:
    return os.path.join(getMainPath(basepath), "input.data")


def getFileOutputPath(basepath: str) -> str:
    return os.path.join(getMainPath(basepath), "output.data")


def hasInitialized(basepath: str)->bool:
    return os.path.exists(getMainPath(basepath))


def getFileExt(filename: str) -> str:
    return os.path.splitext(filename)[1][1:]


def getSystemCommand(cmd: str, man=None) -> str:
    if man.defaultShell == None:
        return cmd
    else:
        return " ".join([man.defaultShell, f'"{cmd}"'])


class WorkManagerState(Enum):
    Empty = 0,
    Loaded = 1,
    LoadFailed = 2,


class WorkManager:
    def __init__(self, path: str):
        self.workingDirectory: str = path
        self.executorMap: dict = {}
        self.tempFileFilter: list = []
        self.currentFile: str = None
        self.importedCommand = {}
        self.defaultShell = None
        self.defaultIO = defaultIO
        self.defaultTimeLimit = defaultTimeLimit
        self.state = WorkManagerState.Empty

    def newCode(self, filename):
        ext = getFileExt(filename)
        lang = fileextToLanguage[ext] if ext in fileextToLanguage else None
        dstPath = os.path.join(self.workingDirectory, filename)
        tempPath = None if lang == None else os.path.join(getTemplatePath(
            self.workingDirectory), f"{TEMPLATE_NAME}.{languageToFileext[lang]}")
        if tempPath != None and os.path.exists(tempPath):
            shutil.copyfile(tempPath, dstPath)
        else:
            open(dstPath, "w").close()
        cli.console.write(color.useGreen("+"), filename)

    def clean(self):
        for file in os.listdir(self.workingDirectory):
            for pat in self.tempFileFilter:
                try:
                    if pat == getFileExt(os.path.split(file)[-1]):
                        os.remove(file)
                        cli.console.write(color.useRed("-"), file)
                        break
                except:
                    pass

    def execute(self, io=None, file=None):
        if io == None:
            io = self.defaultIO
        if file == None:
            file = self.currentFile
        errf = color.useRed("×")
        passf = color.useGreen("√")
        try:
            fileNameWithoutExt, fileext = os.path.splitext(file)
            lang = fileextToLanguage[fileext[1:]]
            cmds = self.executorMap[lang]
            formats = {
                "fileName": file,
                "fileNameWithoutExt": fileNameWithoutExt,
                "dir": self.workingDirectory,
            }
            sumStep = len(cmds)
            cli.console.info(f"Running {file}")
            for ind, bcmd in enumerate(cmds):
                cmd, timelimit = None, None
                if not isinstance(bcmd, str):
                    cmd, timelimit = bcmd
                else:
                    cmd, timelimit = bcmd, self.defaultTimeLimit
                _cmd = cmd.format(**formats)
                cli.console.write(
                    "(", color.useCyan(str(ind+1)), f"/{sumStep}) ", _cmd, sep="")
                proc = None
                if ind == sumStep - 1:  # last command
                    cli.console.write("-"*20)
                    proc = subprocess.Popen(getSystemCommand(_cmd, self), cwd=self.workingDirectory,
                                            stdin=None if io[0] == "s" else open(
                                                getFileInputPath(self.workingDirectory), "r"),
                                            stdout=None if io[1] == "s" else open(getFileOutputPath(self.workingDirectory), "w"))
                else:
                    proc = subprocess.Popen(getSystemCommand(
                        _cmd, self), cwd=self.workingDirectory)

                isTimeout = False
                bg_time = time.time()
                try:
                    proc.communicate(timeout=timelimit)
                except subprocess.TimeoutExpired:
                    isTimeout = True
                    proc.terminate()
                ed_time = time.time()
                if ind == sumStep - 1:  # last command
                    cli.console.write("-"*20)
                cli.console.write(
                    "   ->", passf if proc.returncode == 0 else errf, f"{round((ed_time-bg_time)*1000)/1000}s")
                if proc.returncode != 0:
                    cli.console.write(
                        "(", color.useCyan(str(ind+1)), f"/{sumStep}) ", _cmd, " -> ", proc.returncode, split="", end=" ")
                    if isTimeout:
                        cli.console.write(color.useRed("Time out"))
                    else:
                        cli.console.write()
                    return False
        except BaseException as e:
            return False
        return True


def load(basepath: str) -> WorkManager:
    if not hasInitialized(basepath):
        return None
    ret = WorkManager(basepath)
    try:
        with open(getExecutorPath(basepath), "r", encoding='utf-8') as f:
            ret.executorMap = json.loads(f.read())

        with open(getConfigPath(basepath), "r", encoding='utf-8') as f:
            config = json.loads(f.read())
            ret.tempFileFilter = config[CONST_tempFileFilter]
            ret.importedCommand = config[CONST_importedCommand]
            ret.defaultShell = config[CONST_defaultShell]
            ret.defaultIO = config[CONST_defaultIO]
        ret.state = WorkManagerState.Loaded
    except:
        ret.state = WorkManagerState.LoadFailed
    return ret


def clear(basepath: str):
    oipath = getMainPath(basepath)
    if hasInitialized(basepath):
        shutil.rmtree(oipath)


def initialize(basepath: str):
    clear(basepath)

    oipath = getMainPath(basepath)
    os.mkdir(oipath)

    templatePath = getTemplatePath(basepath)
    os.mkdir(templatePath)
    for k, v in defaultCodeTemplate.items():
        with open(os.path.join(templatePath, f"{TEMPLATE_NAME}.{languageToFileext[k]}"), "w", encoding='utf-8') as f:
            f.write(v)
    executors = defaultExecutors
    with open(getExecutorPath(basepath), "w", encoding='utf-8') as f:
        f.write(json.dumps(executors, indent=4))

    open(getFileInputPath(basepath), "w").close()
    open(getFileOutputPath(basepath), "w").close()

    config = {CONST_tempFileFilter: defaultTempFileFilter,
              CONST_importedCommand: defaultImportedCommand,
              CONST_defaultShell: "powershell -c" if platform.system() == "Windows" else None,
              CONST_defaultIO: defaultIO,
              CONST_defaultTimeLimit: defaultTimeLimit}

    with open(getConfigPath(basepath), "w", encoding='utf-8') as f:
        f.write(json.dumps(config, indent=4))



from prompt_toolkit import HTML

def useRed(content):
    return HTML(f'<obj fg="ansired">{content}</obj>')

def useGreen(content):
    return HTML(f'<obj fg="ansigreen">{content}</obj>')

def useBlue(content):
    return HTML(f'<obj fg="ansiblue">{content}</obj>')

def useCyan(content):
    return HTML(f'<obj fg="ansicyan">{content}</obj>')

def useYellow(content):
    return HTML(f'<obj fg="ansiyellow">{content}</obj>')


from os import path
import pytest
import sys

def main():
    return  # Test is not enable because we use prompt_toolkit
    
    if "quiet" in sys.argv:
        pytest.main(["-q",
                     path.join(path.dirname(__file__), "unit")
                     ])
    else:
        pytest.main([
            path.join(path.dirname(__file__), "unit")
        ])


if __name__ == "__main__":
    main()



import os
import codecs
from setuptools import find_packages, setup
import sys

here = os.path.abspath(os.path.dirname(__file__))


def read(*parts):
    with codecs.open(os.path.join(here, *parts), 'r') as fp:
        return fp.read()


long_description = read('README.md')

setup(
    name="edl-cr",
    version="0.0.1.4",
    description="A CLI tool to run code",
    long_description=long_description,
    long_description_content_type="text/markdown",

    license="Apache-2.0",
    classifiers=[
        "Development Status :: 2 - Pre-Alpha",
        "Programming Language :: Python :: 3.7",
        "License :: OSI Approved :: Apache Software License",
    ],
    url="https://github.com/eXceediDeaL/edl-coderunner",
    project_urls={
        "Bug Tracker": "https://github.com/eXceediDeaL/edl-coderunner/issues",
        "Documentation": "https://github.com/eXceediDeaL/edl-coderunner/wiki",
        "Source Code": "https://github.com/eXceediDeaL/edl-coderunner",
    },
    keywords="code runner",

    author="eXceed-iDeaL",
    author_email="exceedideal@163.com",

    package_dir={"": "src"},
    packages=find_packages(
        where="src",
        exclude=[],
    ),
    package_data={},
    entry_points={
        "console_scripts": [
            "ecr = ecr.__main__:outmain",
        ],
    },

    zip_safe=False,
    python_requires='>=3.7',
    install_requires=[
        "prompt_toolkit>=2.0.7",
        "colorama>=0.3.9"
    ]
)
